;========================================
; Lazarian cartridge game
; Copyrigth 1983 Commodore Inc.
;
; This is the original engine that is 
; adapted to generate a SID file (and a 
; demo PRG)
;
; this version reproduce the music we earth in the various
; screen, according to this information:
;     screen 1: 01  02-
;     screen 2: 00  00-
; sub screen 3: 03  03-
; sub screen 4: 03  03-
; sub screen 5: 03  03-
; sub screen 6: 03  03-
;     screen 7: 01  01-
;     screen 8: 00  00-
; next is screen1 but start with 02 directly
;
; where first number is the startup number of song, the 
; second number is the song to repeat
; this not respect the music we hearth in the game screens.
; So, only screen 1,3 and 7 are reproduced
;
; The original code use NMI for updating the  songs to play,
; while we use the IRQ to allow SID file.

.ifdef sid
 .byte "PSID"
 .word $0200         ; version 2
 .word $7C00         ; data offset
 .word $0000         ; load address in cbm format
 .byte >initMusic
 .byte <initMusic 
 .byte >irqRoutine   ; playmusic
 .byte <irqRoutine   ; playmusic 
 .word $0f00         ; 3 song
 .word $0100         ; default song
 .word $0000
 .word $0000
 .byte "Lazarian",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 .byte "<?>",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 .byte "1983 Commodore Inc",0,0,0,0,0,0,0,0,0,0,0,0,0,0
 .word $0000
 .word $0000
 .word $0000
.endif

.org $8001

 ;=================================
 ; PRG start address
 ;=================================
 .byte $01,$08

 ;=================================
 ; BASIC startup program
 ;=================================
 .byte $0b,$08,$e8,$03,$9e,"2061",0,0,0

.org 2061

resetHandler:
initMusic:
.ifdef sid
      pha                         ; save song number
.else
      sei
;      lda  #$34
;      sta  $0318                  ; Vector: Not maskerable Interrupt (NMI)
;      lda  #$80
;      sta  $0319                  ; Vector: Not maskerable Interrupt (NMI)
      lda  #<irqRoutine
      sta  $0314                  ; Vector: Hardware Interrupt (IRQ)
      lda  #>irqRoutine
      sta  $0315                  ; Vector: Hardware Interrupt (IRQ)
      
      ldx  #$00
      stx  $DC0E       ; Control register A of CIA #1
      inx
      stx  $D01A       ; IRQ mask register
      cli
.endif
      
      lda  #$00
      tay
      
loop1:
      sta  $0002,y
      sta  $D400,y                ; Voice 1: Frequency control (lo byte)
      iny
      bne  loop1

      ;lda  #$01
      ;sta  $D019                  ; raster irq
      ;lda  #$82
      ;sta  $D012       ; Reading/Writing IRQ balance value
      ;lda  #$1B
      ;sta  $D011       ; VIC control register
      

      lda  #$F0
      sta  $D406                  ; Generator 1: Sustain/Release
      sta  $D40D                  ; Generator 2: Sustain/Release
      
      ;lda  $D012
      LDA  #$0F        
      STA  $D418       ;Select volume and filter mode
      
.ifdef sid
      pla
      sta  $23                    ; address like in the game 
      tax 
      
      cmp #$03
      bmi playTune

                                  ; sound a effect
      dex
      dex
      stx $85
      jmp endd
      
playTune:
      cmp #$00
      beq inc1
      
      cmp #$02
      beq dec1
      inx
inc1:
      inx
ends:
      stx $84
endd:
      rts
dec1: dex
      jmp ends
      
.else
                                  ; change manually for different 
				  ; tune or sound effect
      lda  #$00
      sta  $84 
      
      lda  #$05;b
      sta  $85 
      
aaa:
      jmp  aaa
.endif      

nmiLike:
      lda  $84
      bne  endNmi

      lda  $23
      cmp  #$00
      beq  is0
     
      cmp  #$01
      beq  is1
     
      cmp  #$02
      beq  is2
      rts
     
is0:
      lda  #$02
      sta  $84
      rts
is1:
      lda  #$03
      sta  $84
      rts
      
is2:
      lda  #$01
      sta  $84
endNmi:      
      rts
              
initSound:
      stx  $EB         
      lda  control3-1,x           ; load control register for this effect
      sta  $D1
      sta  $D412                  ; Voice 3: Control registers
      lda  lenNote3-1,x           ; load notes duration                
      sta  $D2
      lda  patLo3-1,x             ; load pattern lo pointer
      sta  $CF
      lda  patHi3-1,x             ; load pattern lo pointer
      sta  $D0
      lda  AD3-1,x     
      sta  $D413                  ; Generator 3: Attack/Decay
      lda  SR3-1,x     
      sta  $D414                  ; Generator 3: Sustain/Release
      lda  waveHi3-1,x     
      sta  $D411                  ; Voice 3: Wave form pulsation amplitude (hi byte)
      lda  reson3-1,x     
      sta  $D417                  ; Filter resonance control/voice input control
      lda  volFilter3-1,x         ; load volume and filter mode
      sta  $D418                  ; Select volume and filter mode
      lda  filterHi3-1,x          ; load filter hi for this effect
      sta  $D416                  ; Filter cut frequency: hi byte
      lda  #$01
      sta  $D3
      rts
             
; Parameter for effect sound on voice 3 
control3:
  .byte $10, $80, $20, $40 
  .byte $80, $80, $40, $20
  .byte $20, $40, $40, $80
lenNote3:
  .byte $03, $01, $01, $02
  .byte $02, $02, $02, $01
  .byte $01, $02, $01, $04
patLo3:
  .byte <sound1, <sound2, <sound3, <sound4
  .byte <sound5, <sound6, <sound7, <sound8
  .byte <sound9, <sound10, <sound11, <sound6
patHi3:
  .byte >sound1, >sound2, >sound3, >sound4
  .byte >sound5, >sound6, >sound7, >sound8
  .byte >sound9, >sound10, >sound11, >sound6
AD3:
  .byte $55, $00, $32, $55
  .byte $00, $00, $73, $73
  .byte $32, $55, $40, $00
SR3:
  .byte $A5, $F0, $AA, $A5
  .byte $F0, $F8, $88, $88
  .byte $AA, $A5, $AF, $FA
waveHi3:
  .byte $00, $00, $00, $08
  .byte $00, $00, $06, $00
  .byte $00, $08, $03, $00
reson3:  
  .byte $00, $F4, $00, $00
  .byte $F4, $F4, $00, $00
  .byte $00, $00, $00, $F4
volFilter3:
  .byte $0F, $1F, $0F, $0F
  .byte $1F, $1F, $0F, $0F
  .byte $0F, $0F, $0F, $1F
filterHi3:
  .byte $00, $A2, $00, $00
  .byte $92, $82, $00, $00
  .byte $00, $00, $00, $92
  
;==========================================================
; Voice 3 sounds:
;
; $85      sound to play 
; $CF      pattern pointer lo 
; SD0      pattern pointer hi
; $D1      control register
; $D2      note length for this sound
; $D3      actual note length

;===========================================================
; $82:    next pattern to read index (0=no song or new song)
; $83:    actual note length for voice 1
; $84:    song to play (0=current)
; $86:    actual note length for voice 2
; $87:    freq lo of note
; $88:    freq hi of note
; $8A:    octave for voice 1 note
; $8B:    octave for voice 2 note
; $8C     songs pointer to pattern lo
; $8D     songs pointer to pattern hi

irqRoutine:
.ifndef sid 
     lda  #$01
     sta  $D019       ; Interrupt indicator register
     lda  #$82
     sta  $D012       ; Reading/Writing IRQ balance value
     lda  #$1B
     sta  $D011       ; VIC control register
     lda  #$01
     sta  $D020       ; Border color
.endif

      jsr nmiLike

      cld
      ;lda  $D019                  ; Interrupt indicator register
      ;sta  $D019                  ; Interrupt indicator register

      ldy  $82                    ; next pattern to read index
      bne  readNext
      
      ldy  $84                    ; song to play (0=current)
      bne  newSong
      jmp  voice3Service          ; voice 3 service

newSong:                          ; start the song in y (1,2,3)
      lda  songLo-1,y             ; songs pointer to pattern lo
      sta  $8C         
      lda  songHi-1,y             ; songs pointer to pattern hi 
      sta  $8D
      lda  #$01                   ; start of pattern
      sta  $82                    ; init pattern pointer   
      lda  #$00
      sta  $84                    ; song already initialized
      sta  $83
      sta  $86
      
readNext:  
      ldy  $82
      lda  ($8C),y                ; read pattern data
      tax              
      cmp  #$FF                   ; end of patter?
      bne  noEndOfPattern

endOfPattern:
      lda  $83                    ; read actual note length voice 1
      bne  nextCycle
      lda  $86                    ; read actual note length voice 2
      bne  nextCycle
      jsr  stopSound1_2
      jmp  voice3Service

noEndOfPattern:
      cmp  #$FD         
      bne  noFD

isFD:
      iny              
      lda  ($8C),y                ; read next pattern data
      sta  $8A                    ; octave for voice 1
      jmp  incIndex

noFD:
      cmp  #$FE        
      bne  noFE
      
isFE: 
      iny              
      lda  ($8C),y                ; read next pattern data
      sta  $8B                    ; octave for voice 2
      
incIndex:
      inc  $82
      inc  $82
      jmp  readNext

noFE:
      txa              
      bmi  playNote2              ; is for voice 2?
      
playNote1:
      ldy  $83                    ; actual note length
      bne  nextCycle
      ldy  $8A                    ; load octave for this note
      jsr  calcNote
      inc  $82
      sta  $D400                  ; Voice 1: Frequency control (lo byte)
      stx  $D401                  ; Voice 1: Frequency control (hi byte)
      tya                         ; y=note length
      ldy  $E4
      beq  skip1
      sec              
      sbc  #$02
      
skip1:
      sta  $83                    ; store note length
      lda  #$11        
      sta  $D404                  ; Voice 1: Control registers
      bne  readNext

playNote2:
      ldy  $86                    ; actual note length
      bne  nextCycle                  
      
      ldy  $8B                    ; load octave for this note
      jsr  calcNote
      inc  $82
      sta  $D407                  ; Voice 2: Frequency control (lo byte)
      stx  $D408                  ; Voice 2: Frequency control (hi byte)
      tya              
      ldy  $E4
      beq  skip2
      sec              
      sbc  #$02
        
skip2:
      sta  $86                    ; store note length
      lda  #$11        
      sta  $D40B                  ; Voice 2: Control registers
      bne  readNext

nextCycle:
      lda  $83                    ; read note length for voice 1
      beq  noEndNote1             ; end of duration?
      dec  $83
      bne  noEndNote1             ; end of duration?
      lda  #$00        
      sta  $D404                  ; Voice 1: Control registers

noEndNote1:
      lda  $86                    ; read note length for voice 2
      beq  voice3Service          ; end of duration?
      dec  $86
      bne  voice3Service          ; end of duration?
      lda  #$00        
      sta  $D40B                  ; Voice 2: Control registers

voice3Service:
      lda  $D0                    ; pointer to 0 -> we aren't sounding
      bne  soundIt
      ldx  $85                    ; sound to play (0=no)
      beq  endService
      jsr  initSound
      lda  #$00
      sta  $85                    ; sound just initialized 

soundIt:
      lda  $D3                    ; actual note length
      beq  readNext3              ; note finished?
      dec  $D3                    
      jmp  endService

readNext3:
      ldy  #$00        
      lda  ($CF),y                ; read next pattern of voice 3
      cmp  #$FF
      bne  isFreq
      iny              
      lda  ($CF),y                ; read next pattern of voice 3
      cmp  #$FF        
      beq  stopVoice3
      ora  $D1
      sta  $D412                  ; Voice 3: Control registers
      jmp  adjPatPoint

stopVoice3:
      lda  #$00        
      sta  $D412                  ; Voice 3: Control registers
      sta  $CF   
      sta  $D0                    ; mark to no sound at next irq
      sta  $EB
      beq  endService
      
isFreq:                           ; the readed byte is freq value
      sta  $D40F                  ; Voice 3: Frequency control (hi byte)
      iny              
      lda  ($CF),y                ; read next pattern of voice 3
      sta  $D40E                  ; Voice 3: Frequency control (lo byte)
      lda  $D2                    ; note length
      sta  $D3                    ; actual note length
      
adjPatPoint:
      lda  $CF
      clc
      adc  #$02
      sta  $CF                    ; adj pattern pointer 
      bcc  endService
      inc  $D0

endService:
;      lda  $BA
;      beq  $9F27
;      lda  $ED
;      eor  #$FF
;      sta  $ED
;      beq  $9F27
;      dec  $BA
;      lda  $BA
;      beq  $9F21
;      and  #$03
;      tax
;      lda  $A093,x
;9F21  sta  $D020                  ; Border color
;      sta  $D021                  ; Background 0 color
;9F27
      ;pla
      ;tay
      ;pla
      ;tax
      ;pla
      
.ifdef sid
      rts
.else
      dec $d020
      jmp $EA31
.endif


;=====================================
; Calculate note freq/duration
;=====================================

calcNote:
      sty  $89
      tay
      and  #$0F
      tax
      lda  freqLo,x               ; load freq. lo of this note (x)
      sta  $87                    ; store freq. lo
      lda  freqHi,x               ; load freq. hi of this note (x)
      sta  $88                    ; store freq. hi
      sec
      lda  #$08
      sbc  $89
      beq  calcLen
      tax                         ; change octave of note
      lda  $88
loopF:
      lsr  a
      ror  $87
      dex
      bne  loopF
      sta  $88

calcLen:                          ; calculate lenght of note 
      tya
      lsr  a
      lsr  a
      lsr  a
      lsr  a
      and  #$07
      tax
      lda  lenTable,x
      tay
      ldx  $88                    ; load freq. hi of this note 
      lda  $87                    ; load freq. lo of this note 
      rts

;=================================
; Stop to sound for voice 1 and 2
;=================================
stopSound1_2:
      lda  #$00        
      sta  $D404                  ; Voice 1: Control registers
      sta  $D40B                  ; Voice 2: Control registers
      sta  $82
      rts
              
lenTable:
  .byte $04, $08, $0C, $10
  .byte $18, $20, $30, $40

freqLo:
  .byte $00                       ; no sound
  .byte $7A                       ; C
  .byte $79                       ; C#
  .byte $F2                       ; D
  .byte $EC                       ; D#
  .byte $6E                       ; E
  .byte $82                       ; F
  .byte $2E                       ; F#
  .byte $7D                       ; G
  .byte $78                       ; G#
  .byte $2A                       ; D#
  .byte $9D                       ; A 
  .byte $DC                       ; A#
 
freqHi:
  .byte $00                       ; no sound
  .byte $86
  .byte $8E
  .byte $96
  .byte $9F
  .byte $A9
  .byte $B3
  .byte $BE
  .byte $C9
  .byte $D5
  .byte $E2
  .byte $EF
  .byte $FD
  
  
;===============================
; pattern of note:
;
; FD xx      octave for voice 1 note 
; FE xx      octave for voice 2 note
; FF         end of pattern
; mn         n=note  m=vkkk  
;            v=0 for voice 1
;	    v=1 for voice 2
;	    kkk=codified length
;=============================== 
  
song1:
  .byte $FD, $05
  .byte $FE, $04
  .byte $35, $B1, $45, $C1
  .byte $35, $B1, $48, $C5
  .byte $38, $B5, $46, $C3
  .byte $35, $B1, $43, $C6
  .byte $31, $B5, $41, $C5
  .byte $33, $B6, $45, $C8
  .byte $35, $B1, $43, $C6
  .byte $31, $B5, $43, $C6
  .byte $35, $B1, $45, $C8
  .byte $35, $B1, $48, $C5
  .byte $38, $B5, $46, $C3
  .byte $35, $B1, $43, $C6
  .byte $31, $B5, $41, $C5
  .byte $33, $B6, $45, $C1
  .byte $35, $B1, $43, $C6
  .byte $41, $CA, $40, $C0
  .byte $FF

song2:  
  .byte $FD, $05
  .byte $FE, $04
  .byte $35, $F1, $35, $36
  .byte $38, $F3, $38, $36
  .byte $35, $33, $D5, $31
  .byte $31, $D6, $33, $35
  .byte $D8, $45, $13, $D5
  .byte $53, $F1, $35, $35
  .byte $36, $38, $F3, $38
  .byte $36, $35, $33, $D5
  .byte $31, $31, $D6, $33
  .byte $35, $D8, $43, $11
  .byte $D5, $51, $F8, $33
  .byte $33, $35, $31, $F8
  .byte $33, $15, $16, $35
  .byte $31, $D8, $33, $15
  .byte $16, $D9, $35, $33
  .byte $BA, $31, $B7, $33, $D8
  .byte $FD, $04
  .byte $55
  .byte $FD, $05
  .byte $F1, $35, $35, $36
  .byte $38, $F3, $38, $36
  .byte $35, $33, $D5, $31
  .byte $31, $D6, $33, $35
  .byte $D8, $43, $11, $D5
  .byte $51, $30
  .byte $FF

song3:
  .byte $FD, $04
  .byte $FE, $04
  .byte $99, $1C, $99, $1C
  .byte $A9, $2C, $99, $1C
  .byte $99, $1C, $A9, $2C
  .byte $99, $1C, $99, $1C 
  .byte $FD, $05
  .byte $A9, $25, $A9, $27, $A9, $29
  .byte $FD, $04
  .byte $99, $1C, $99, $1C, $A9
  .byte $2C, $99, $1C, $99, $1C
  .byte $FD, $05
  .byte $A9, $25, $A9, $29
  .byte $A7, $27, $A7, $24
  .byte $FD, $04
  .byte $A7, $2C, $99, $1C, $99, $1C
  .byte $A9, $2C, $99, $1C, $99, $1C
  .byte $A9, $2C, $99, $1C, $99, $1C
  .byte $FD, $05
  .byte $A9, $25, $A9, $27
  .byte $A9, $29, $A0, $15
  .byte $19, $4C, $A9, $99
  .byte $A9, $1A, $19, $A9
  .byte $17, $A9, $25, $A5
  .byte $29, $A9, $25
  .byte $FF

;A093:
;  .byte $04, $00, $02, $06
 
songLo:
  .byte <song1-1, <song2-1, <song3-1
  
songHi:  
  .byte >song1, >song2, >song3
  
  
;==================================
; FF xx
; hh ll      hh=freq hi  ll=freq lo
; FF FF      end of pattern
;==================================
  
sound1:
  .byte $FF, $01
  .byte $2F, $44
  .byte $FF, $00
  .byte $2F, $44
  .byte $FF, $FF
sound2:  
  .byte $FF, $01
  .byte $12, $00, $12, $FA
  .byte $13, $F4, $14, $EE
  .byte $15, $E8, $15, $E8
  .byte $FF, $00
  .byte $15, $E8, $15, $E8
  .byte $15, $E8, $12, $00
  .byte $FF, $FF
sound3:
  .byte $FF, $01
  .byte $0E, $00, $10, $58
  .byte $0E, $00, $10, $58
  .byte $0E, $00
  .byte $FF, $00
  .byte $10, $58, $0E, $00
  .byte $10, $58, $0E, $00
  .byte $10, $58
  .byte $FF, $FF
sound4:
  .byte $FF, $01
  .byte $3E, $E4, $00, $00
  .byte $3E, $E4, $00, $00
  .byte $3E, $E4, $00, $00
  .byte $3E, $E4
  .byte $FF, $00
  .byte $3E, $E4, $00, $00
  .byte $3E, $E4, $00, $00
  .byte $3E, $E4, $00, $00
  .byte $FF, $FF
sound5: 
  .byte $FF, $01
  .byte $0C, $00, $0C, $00
  .byte $0C, $00, $0C, $00
  .byte $0C, $00
  .byte $FF, $00
  .byte $0C, $00
  .byte $FF, $FF
sound6: 
  .byte $FF, $01
  .byte $05, $F4, $07, $E8
  .byte $09, $DC, $0B, $D0
  .byte $0D, $C4, $0D, $C4
  .byte $0D, $C4, $0D, $C4
  .byte $0D, $C4, $0D, $C4
  .byte $0D, $C4, $0D, $C4
  .byte $0D, $C4, $0D, $C4
  .byte $0D, $C4
  .byte $FF, $00
  .byte $11, $AC, $13, $A0
  .byte $15, $94, $19, $7C
  .byte $1B, $70, $19, $7C
  .byte $1B, $70, $19, $7C
  .byte $1B, $70, $19, $7C
  .byte $1B, $70, $19, $7C
  .byte $1B, $70, $19, $7C
  .byte $1B, $70, $19, $7C
  .byte $1B, $70, $1B, $70
  .byte $1B, $70
  .byte $FF, $FF
sound7:
  .byte $FF, $01
  .byte $0F, $E8, $13, $D0
  .byte $15, $C4, $15, $C4
  .byte $FF, $00
  .byte $15, $C4, $15, $C4
  .byte $15, $C4, $15, $C4
  .byte $FF, $FF
sound8:
  .byte $FF, $01
  .byte $0A, $00, $0B, $F4
  .byte $0D, $E8, $0F, $DC
  .byte $FF, $00
  .byte $0F, $DC, $0F, $DC
  .byte $0F, $DC, $0F, $DC
  .byte $FF, $FF
sound9:
  .byte $FF, $01
  .byte $0B, $B8, $0D, $AC
  .byte $0B, $B8, $0D, $AC
  .byte $0B, $B8
  .byte $FF, $00
  .byte $0D, $AC, $0B, $B8
  .byte $0D, $AC, $0B, $B8
  .byte $0D, $AC, $0B, $B8
  .byte $0D, $AC, $0B, $B8
  .byte $FF, $FF
sound10: 
  .byte $FF, $01
  .byte $2F, $44, $00, $00
  .byte $2F, $44, $00, $00
  .byte $2F, $44
  .byte $FF, $00
  .byte $2F, $44, $00, $00
  .byte $2F, $44, $00, $00
  .byte $2F, $44
  .byte $FF, $FF
sound11:
  .byte $FF, $01
  .byte $2E, $E0, $2C, $EC
  .byte $2A, $F8, $29, $04
  .byte $FF, $00
  .byte $2E, $E0, $2C, $EC
  .byte $2A, $F8       
  .byte $FF, $01
  .byte $29, $04, $2E, $E0
  .byte $2C, $EC, $2A, $F8
  .byte $FF, $00
  .byte $2A, $F8, $29, $04
  .byte $2E, $E0
  .byte $FF, $01
  .byte $2C, $EC, $2A, $F8
  .byte $29, $04, $2E, $E0
  .byte $FF, $00
  .byte $2C, $EC, $2A, $F8
  .byte $29, $04
  .byte $FF, $FF
 
